# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .cohort_response import CohortResponse
from .lang2fhir_and_create_multi_response import Lang2FhirAndCreateMultiResponse
from .lang2fhir_and_create_multi_response_resource_info_item import Lang2FhirAndCreateMultiResponseResourceInfoItem
from .lang2fhir_and_create_multi_response_response_bundle import Lang2FhirAndCreateMultiResponseResponseBundle
from .lang2fhir_and_create_request_resource import Lang2FhirAndCreateRequestResource
from .lang2fhir_and_create_response import Lang2FhirAndCreateResponse
from .lang2fhir_and_search_response import Lang2FhirAndSearchResponse
from .mcp_server_response import McpServerResponse
from .mcp_server_response_data import McpServerResponseData
from .mcp_server_tool_call_response import McpServerToolCallResponse
from .mcp_server_tool_response import McpServerToolResponse
from .mcp_server_tool_response_data import McpServerToolResponseData
from .search_concept import SearchConcept

__all__ = [
    "CohortResponse",
    "Lang2FhirAndCreateMultiResponse",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem",
    "Lang2FhirAndCreateMultiResponseResponseBundle",
    "Lang2FhirAndCreateRequestResource",
    "Lang2FhirAndCreateResponse",
    "Lang2FhirAndSearchResponse",
    "McpServerResponse",
    "McpServerResponseData",
    "McpServerToolCallResponse",
    "McpServerToolResponse",
    "McpServerToolResponseData",
    "SearchConcept",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .create_multi_response_bundle_entry_item import CreateMultiResponseBundleEntryItem


class CreateMultiResponseBundle(UniversalBaseModel):
    """
    FHIR transaction Bundle containing all extracted resources
    """

    resource_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="resourceType")] = None
    type: typing.Optional[str] = None
    entry: typing.Optional[typing.List[CreateMultiResponseBundleEntryItem]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .decision_node_definition import DecisionNodeDefinition
from .lang2fhir_create_definition import Lang2FhirCreateDefinition
from .lang2fhir_search_definition import Lang2FhirSearchDefinition
from .sub_workflow_definition import SubWorkflowDefinition


class StepOperation(UniversalBaseModel):
    search_definition: typing.Optional[Lang2FhirSearchDefinition] = None
    create_definition: typing.Optional[Lang2FhirCreateDefinition] = None
    workflow_definition: typing.Optional[SubWorkflowDefinition] = None
    decision_definition: typing.Optional[DecisionNodeDefinition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CohortResponse,
    Lang2FhirAndCreateMultiResponse,
    Lang2FhirAndCreateMultiResponseResourceInfoItem,
    Lang2FhirAndCreateMultiResponseResponseBundle,
    Lang2FhirAndCreateRequestResource,
    Lang2FhirAndCreateResponse,
    Lang2FhirAndSearchResponse,
    McpServerResponse,
    McpServerResponseData,
    McpServerToolCallResponse,
    McpServerToolResponse,
    McpServerToolResponseData,
    SearchConcept,
)
from .errors import BadRequestError, FailedDependencyError, ForbiddenError, InternalServerError, UnauthorizedError
from . import mcp_server

__all__ = [
    "BadRequestError",
    "CohortResponse",
    "FailedDependencyError",
    "ForbiddenError",
    "InternalServerError",
    "Lang2FhirAndCreateMultiResponse",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem",
    "Lang2FhirAndCreateMultiResponseResponseBundle",
    "Lang2FhirAndCreateRequestResource",
    "Lang2FhirAndCreateResponse",
    "Lang2FhirAndSearchResponse",
    "McpServerResponse",
    "McpServerResponseData",
    "McpServerToolCallResponse",
    "McpServerToolResponse",
    "McpServerToolResponseData",
    "SearchConcept",
    "UnauthorizedError",
    "mcp_server",
]



# This file was auto-generated by Fern from our API Definition.

import typing

ExtractRequestConfigChunkingMethod = typing.Union[
    typing.Literal["none", "sentences", "paragraphs", "topics"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawConstrueClient, RawConstrueClient
from .types.construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .types.extract_codes_result import ExtractCodesResult
from .types.extract_request_config import ExtractRequestConfig
from .types.extract_request_system import ExtractRequestSystem
from .types.upload_request_format import UploadRequestFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConstrueClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConstrueClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConstrueClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConstrueClient
        """
        return self._raw_client

    def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConstrueUploadCodeSystemResponse:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConstrueUploadCodeSystemResponse
            Successfully uploaded code system

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.construe.upload_code_system(
            name="CUSTOM_CODES",
            version="1.0",
            format="json",
            file="file",
        )
        """
        _response = self._raw_client.upload_code_system(
            name=name,
            version=version,
            format=format,
            file=file,
            revision=revision,
            code_col=code_col,
            desc_col=desc_col,
            defn_col=defn_col,
            request_options=request_options,
        )
        return _response.data

    def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractCodesResult:
        """
        Converts natural language text into structured medical codes

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractCodesResult
            Successfully extracted codes

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.construe.extract_codes(
            text="Patient is a 14-year-old female, previously healthy, who is here for evaluation of abnormal renal ultrasound with atrophic right kidney",
        )
        """
        _response = self._raw_client.extract_codes(
            text=text, system=system, config=config, request_options=request_options
        )
        return _response.data


class AsyncConstrueClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConstrueClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConstrueClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConstrueClient
        """
        return self._raw_client

    async def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConstrueUploadCodeSystemResponse:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConstrueUploadCodeSystemResponse
            Successfully uploaded code system

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.construe.upload_code_system(
                name="CUSTOM_CODES",
                version="1.0",
                format="json",
                file="file",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_code_system(
            name=name,
            version=version,
            format=format,
            file=file,
            revision=revision,
            code_col=code_col,
            desc_col=desc_col,
            defn_col=defn_col,
            request_options=request_options,
        )
        return _response.data

    async def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractCodesResult:
        """
        Converts natural language text into structured medical codes

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractCodesResult
            Successfully extracted codes

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.construe.extract_codes(
                text="Patient is a 14-year-old female, previously healthy, who is here for evaluation of abnormal renal ultrasound with atrophic right kidney",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extract_codes(
            text=text, system=system, config=config, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Lang2FhirAndSearchResponse(UniversalBaseModel):
    resource_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The FHIR resource type identified for the search
    """

    search_params: typing.Optional[str] = pydantic.Field(default=None)
    """
    FHIR search parameters in standard format
    """

    fhir_results: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(
        default=None
    )
    """
    Array of FHIR resources returned from the search
    """

    success: typing.Optional[bool] = None
    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class WorkflowConfig(UniversalBaseModel):
    fhir_provider_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    FHIR provider IDs for executing workflow steps
    """

    dynamic_generation: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to use dynamic lang2fhir generation instead of pre-populated templates
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

Role = typing.Union[
    typing.Literal[
        "SmartV1Admin",
        "SmartV1Read",
        "SmartV1Write",
        "SmartV2Admin",
        "SmartV2Read",
        "SmartV2Write",
        "USCDISmartV1Admin",
        "USCDISmartV1Read",
        "USCDISmartV1Write",
        "USCDISmartV2Admin",
        "USCDISmartV2Read",
        "USCDISmartV2Write",
        "CernerSmartV1Admin",
        "CernerSmartV1Read",
        "CernerSmartV1Write",
        "CernerSmartV2Admin",
        "CernerSmartV2Read",
        "CernerSmartV2Write",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ChatSessionTemplate(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session ID
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    User ID
    """

    session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session ID
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session status
    """

    system_prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    System prompt
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session title
    """

    agent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Agent ID
    """

    created: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session created time
    """

    last_message_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session last message time
    """

    updated: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session updated time
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ServiceAccountKey(UniversalBaseModel):
    """
    Google Cloud Service Account key (required for google_healthcare auth method)
    """

    type: str
    project_id: str
    private_key_id: str
    private_key: str
    client_email: str
    client_id: str
    auth_uri: str
    token_uri: str
    auth_provider_x509cert_url: typing_extensions.Annotated[str, FieldMetadata(alias="auth_provider_x509_cert_url")]
    client_x509cert_url: typing_extensions.Annotated[str, FieldMetadata(alias="client_x509_cert_url")]
    universe_domain: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.agent_prompts_response import AgentPromptsResponse
from ..types.json_patch import JsonPatch
from ..types.success_response import SuccessResponse
from .types.prompts_delete_response import PromptsDeleteResponse
from .types.prompts_list_response import PromptsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPromptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        name: str,
        content: str,
        is_active: bool,
        description: typing.Optional[str] = OMIT,
        is_default: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentPromptsResponse]:
        """
        Creates a new agent prompt

        Parameters
        ----------
        name : str
            Prompt name

        content : str
            Prompt content

        is_active : bool
            Whether the prompt is active

        description : typing.Optional[str]
            Prompt description

        is_default : typing.Optional[bool]
            Whether this is a default prompt

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorizing the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentPromptsResponse]
            Prompt created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "agent/prompts",
            method="POST",
            json={
                "name": name,
                "description": description,
                "content": content,
                "is_default": is_default,
                "is_active": is_active,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[PromptsListResponse]:
        """
        Retrieves a list of agent prompts belonging to the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PromptsListResponse]
            Prompts retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "agent/prompts/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PromptsListResponse,
                    parse_obj_as(
                        type_=PromptsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AgentPromptsResponse]:
        """
        Retrieves a specific prompt by its ID

        Parameters
        ----------
        id : str
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentPromptsResponse]
            Prompt retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        is_default: typing.Optional[bool] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentPromptsResponse]:
        """
        Updates an existing prompt

        Parameters
        ----------
        id : str
            Prompt ID

        name : typing.Optional[str]
            Prompt name

        description : typing.Optional[str]
            Prompt description

        content : typing.Optional[str]
            Prompt content

        is_default : typing.Optional[bool]
            Whether this is a default prompt

        is_active : typing.Optional[bool]
            Whether the prompt is active

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorizing the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentPromptsResponse]
            Prompt updated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "content": content,
                "is_default": is_default,
                "is_active": is_active,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PromptsDeleteResponse]:
        """
        Soft deletes a prompt by setting is_active to false

        Parameters
        ----------
        id : str
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PromptsDeleteResponse]
            Prompt deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PromptsDeleteResponse,
                    parse_obj_as(
                        type_=PromptsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch(
        self, id: str, *, request: JsonPatch, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AgentPromptsResponse]:
        """
        Patches an existing prompt

        Parameters
        ----------
        id : str
            Agent Prompt ID

        request : JsonPatch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentPromptsResponse]
            Prompt patched successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=JsonPatch, direction="write"),
            headers={
                "content-type": "application/json+patch",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def load_defaults(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SuccessResponse]:
        """
        Loads default agent prompts for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SuccessResponse]
            Default prompts loaded successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "agent/prompts/load-defaults",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SuccessResponse,
                    parse_obj_as(
                        type_=SuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPromptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        name: str,
        content: str,
        is_active: bool,
        description: typing.Optional[str] = OMIT,
        is_default: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentPromptsResponse]:
        """
        Creates a new agent prompt

        Parameters
        ----------
        name : str
            Prompt name

        content : str
            Prompt content

        is_active : bool
            Whether the prompt is active

        description : typing.Optional[str]
            Prompt description

        is_default : typing.Optional[bool]
            Whether this is a default prompt

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorizing the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentPromptsResponse]
            Prompt created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agent/prompts",
            method="POST",
            json={
                "name": name,
                "description": description,
                "content": content,
                "is_default": is_default,
                "is_active": is_active,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PromptsListResponse]:
        """
        Retrieves a list of agent prompts belonging to the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PromptsListResponse]
            Prompts retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agent/prompts/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PromptsListResponse,
                    parse_obj_as(
                        type_=PromptsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AgentPromptsResponse]:
        """
        Retrieves a specific prompt by its ID

        Parameters
        ----------
        id : str
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentPromptsResponse]
            Prompt retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        is_default: typing.Optional[bool] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentPromptsResponse]:
        """
        Updates an existing prompt

        Parameters
        ----------
        id : str
            Prompt ID

        name : typing.Optional[str]
            Prompt name

        description : typing.Optional[str]
            Prompt description

        content : typing.Optional[str]
            Prompt content

        is_default : typing.Optional[bool]
            Whether this is a default prompt

        is_active : typing.Optional[bool]
            Whether the prompt is active

        tags : typing.Optional[typing.Sequence[str]]
            Tags for categorizing the prompt

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentPromptsResponse]
            Prompt updated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "content": content,
                "is_default": is_default,
                "is_active": is_active,
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PromptsDeleteResponse]:
        """
        Soft deletes a prompt by setting is_active to false

        Parameters
        ----------
        id : str
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PromptsDeleteResponse]
            Prompt deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PromptsDeleteResponse,
                    parse_obj_as(
                        type_=PromptsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch(
        self, id: str, *, request: JsonPatch, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AgentPromptsResponse]:
        """
        Patches an existing prompt

        Parameters
        ----------
        id : str
            Agent Prompt ID

        request : JsonPatch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentPromptsResponse]
            Prompt patched successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"agent/prompts/{jsonable_encoder(id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=JsonPatch, direction="write"),
            headers={
                "content-type": "application/json+patch",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentPromptsResponse,
                    parse_obj_as(
                        type_=AgentPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def load_defaults(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SuccessResponse]:
        """
        Loads default agent prompts for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SuccessResponse]
            Default prompts loaded successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "agent/prompts/load-defaults",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SuccessResponse,
                    parse_obj_as(
                        type_=SuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AuthMethod,
    FhirProviderAuthConfig,
    FhirProviderDeleteResponse,
    FhirProviderListResponse,
    FhirProviderListResponseFhirProvidersItem,
    FhirProviderRemoveAuthConfigResponse,
    FhirProviderResponse,
    FhirProviderResponseData,
    FhirProviderSandboxInfo,
    FhirProviderTemplate,
    FhirQueryResponse,
    FhirQueryResponseData,
    JsonWebKey,
    Provider,
    Role,
    ServiceAccountKey,
    ServiceAccountMetadata,
    SmartConfiguration,
)
from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError

__all__ = [
    "AuthMethod",
    "BadRequestError",
    "FhirProviderAuthConfig",
    "FhirProviderDeleteResponse",
    "FhirProviderListResponse",
    "FhirProviderListResponseFhirProvidersItem",
    "FhirProviderRemoveAuthConfigResponse",
    "FhirProviderResponse",
    "FhirProviderResponseData",
    "FhirProviderSandboxInfo",
    "FhirProviderTemplate",
    "FhirQueryResponse",
    "FhirQueryResponseData",
    "ForbiddenError",
    "InternalServerError",
    "JsonWebKey",
    "NotFoundError",
    "Provider",
    "Role",
    "ServiceAccountKey",
    "ServiceAccountMetadata",
    "SmartConfiguration",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .fhir_provider_list_response_fhir_providers_item import FhirProviderListResponseFhirProvidersItem


class FhirProviderListResponse(UniversalBaseModel):
    """
    Response payload for listing FHIR Providers.
    On shared instances, only sandbox providers are returned (as FhirProviderSandboxInfo).
    On dedicated instances, full provider details are returned (as FhirProviderTemplate).
    """

    success: typing.Optional[bool] = None
    message: typing.Optional[str] = None
    fhir_providers: typing.Optional[typing.List[FhirProviderListResponseFhirProvidersItem]] = pydantic.Field(
        default=None
    )
    """
    List of FHIR providers. Sandbox providers return FhirProviderSandboxInfo,
    other providers return FhirProviderTemplate.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .chat_message_template_role import ChatMessageTemplateRole


class ChatMessageTemplate(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat message ID
    """

    session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session ID
    """

    role: typing.Optional[ChatMessageTemplateRole] = pydantic.Field(default=None)
    """
    Message role indicating the source/type of the message:
    - `user` - Messages from the user
    - `assistant` - Text responses from the AI assistant
    - `model` - Function/tool call requests to the model
    - `function` - Function/tool call results
    """

    content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message content
    """

    created: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message created time
    """

    updated: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message updated time
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    User ID
    """

    function_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Function name
    """

    function_args: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Function arguments
    """

    function_result: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Function result
    """

    message_order: typing.Optional[int] = pydantic.Field(default=None)
    """
    Message order
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CreateMultiResponse,
    CreateMultiResponseBundle,
    CreateMultiResponseBundleEntryItem,
    CreateMultiResponseBundleEntryItemRequest,
    CreateMultiResponseResourcesItem,
    CreateRequestResource,
    DocumentRequestFileType,
    DocumentRequestResource,
    FhirResource,
    Lang2FhirUploadProfileResponse,
    SearchResponse,
    SearchResponseResourceType,
)
from .errors import BadRequestError, FailedDependencyError, ForbiddenError, InternalServerError, UnauthorizedError

__all__ = [
    "BadRequestError",
    "CreateMultiResponse",
    "CreateMultiResponseBundle",
    "CreateMultiResponseBundleEntryItem",
    "CreateMultiResponseBundleEntryItemRequest",
    "CreateMultiResponseResourcesItem",
    "CreateRequestResource",
    "DocumentRequestFileType",
    "DocumentRequestResource",
    "FailedDependencyError",
    "FhirResource",
    "ForbiddenError",
    "InternalServerError",
    "Lang2FhirUploadProfileResponse",
    "SearchResponse",
    "SearchResponseResourceType",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .errors.bad_request_error import BadRequestError
from .errors.failed_dependency_error import FailedDependencyError
from .errors.forbidden_error import ForbiddenError
from .errors.internal_server_error import InternalServerError
from .errors.unauthorized_error import UnauthorizedError
from .types.create_multi_response import CreateMultiResponse
from .types.create_request_resource import CreateRequestResource
from .types.document_request_file_type import DocumentRequestFileType
from .types.document_request_resource import DocumentRequestResource
from .types.fhir_resource import FhirResource
from .types.lang2fhir_upload_profile_response import Lang2FhirUploadProfileResponse
from .types.search_response import SearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLang2FhirClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        version: str,
        resource: CreateRequestResource,
        text: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FhirResource]:
        """
        Converts natural language text into a structured FHIR resource

        Parameters
        ----------
        version : str
            FHIR version to use

        resource : CreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile. Recommended to use the supported US Core Profiles for validated results but you can also use any custom profile you've uploaded (if you're a develop or launch customer)

        text : str
            Natural language text to convert

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirResource]
            Successfully created FHIR resource
        """
        _response = self._client_wrapper.httpx_client.request(
            "lang2fhir/create",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirResource,
                    parse_obj_as(
                        type_=FhirResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_multi(
        self,
        *,
        text: str,
        version: typing.Optional[str] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateMultiResponse]:
        """
        Analyzes natural language text and extracts multiple FHIR resources, returning them as a transaction Bundle.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types from the text.
        Resources are linked with proper references (e.g., Conditions reference the Patient).

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        version : typing.Optional[str]
            FHIR version to use

        provider : typing.Optional[str]
            Optional FHIR provider name for provider-specific profiles

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateMultiResponse]
            Successfully extracted FHIR resources
        """
        _response = self._client_wrapper.httpx_client.request(
            "lang2fhir/create/multi",
            method="POST",
            json={
                "text": text,
                "version": version,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateMultiResponse,
                    parse_obj_as(
                        type_=CreateMultiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search(
        self, *, text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SearchResponse]:
        """
        Converts natural language text into FHIR search parameters.
        Automatically identifies the appropriate FHIR resource type and generates valid search query parameters.

        Supported resource types include: AllergyIntolerance, Appointment, CarePlan, CareTeam, Condition,
        Coverage, Device, DiagnosticReport, DocumentReference, Encounter, Goal, Immunization, Location,
        Medication, MedicationRequest, Observation, Organization, Patient, PlanDefinition, Practitioner,
        PractitionerRole, Procedure, Provenance, Questionnaire, QuestionnaireResponse, RelatedPerson,
        Schedule, ServiceRequest, Slot, and Specimen.

        Parameters
        ----------
        text : str
            Natural language text to convert into FHIR search parameters.
            The system will automatically identify the appropriate resource type and generate valid search parameters.

            Examples:
            - "Appointments between March 2-9, 2025" → Appointment search with date range
            - "Patients with diabetes" → Condition search with code parameter
            - "Active medication requests for metformin" → MedicationRequest search
            - "Lab results for creatinine" → DiagnosticReport search
            - "Dr. Smith's schedule" → Practitioner or Schedule search

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchResponse]
            Successfully generated FHIR search parameters
        """
        _response = self._client_wrapper.httpx_client.request(
            "lang2fhir/search",
            method="POST",
            json={
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchResponse,
                    parse_obj_as(
                        type_=SearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upload_profile(
        self, *, version: str, resource: str, profile: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Lang2FhirUploadProfileResponse]:
        """
        Upload a custom FHIR StructureDefinition profile for use with the lang2fhir service

        Parameters
        ----------
        version : str
            FHIR version that this profile implements

        resource : str
            Name for the custom resource profile (will be converted to lowercase)

        profile : str
            Base64 encoded JSON string of the FHIR StructureDefinition profile

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Lang2FhirUploadProfileResponse]
            Profile successfully uploaded
        """
        _response = self._client_wrapper.httpx_client.request(
            "lang2fhir/profile/upload",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "profile": profile,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirUploadProfileResponse,
                    parse_obj_as(
                        type_=Lang2FhirUploadProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def document(
        self,
        *,
        version: str,
        resource: DocumentRequestResource,
        content: str,
        file_type: DocumentRequestFileType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FhirResource]:
        """
        Extracts text from a document (PDF or image) and converts it into a structured FHIR resource

        Parameters
        ----------
        version : str
            FHIR version to use

        resource : DocumentRequestResource
            Type of FHIR resource to create (questionnaire and US Core questionnaireresponse profiles currently supported)

        content : str
            Base64 encoded file content

        file_type : DocumentRequestFileType
            MIME type of the file

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirResource]
            Successfully created FHIR resource from document
        """
        _response = self._client_wrapper.httpx_client.request(
            "lang2fhir/document",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "content": content,
                "fileType": file_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirResource,
                    parse_obj_as(
                        type_=FhirResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLang2FhirClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        version: str,
        resource: CreateRequestResource,
        text: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FhirResource]:
        """
        Converts natural language text into a structured FHIR resource

        Parameters
        ----------
        version : str
            FHIR version to use

        resource : CreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile. Recommended to use the supported US Core Profiles for validated results but you can also use any custom profile you've uploaded (if you're a develop or launch customer)

        text : str
            Natural language text to convert

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirResource]
            Successfully created FHIR resource
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lang2fhir/create",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirResource,
                    parse_obj_as(
                        type_=FhirResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_multi(
        self,
        *,
        text: str,
        version: typing.Optional[str] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateMultiResponse]:
        """
        Analyzes natural language text and extracts multiple FHIR resources, returning them as a transaction Bundle.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types from the text.
        Resources are linked with proper references (e.g., Conditions reference the Patient).

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        version : typing.Optional[str]
            FHIR version to use

        provider : typing.Optional[str]
            Optional FHIR provider name for provider-specific profiles

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateMultiResponse]
            Successfully extracted FHIR resources
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lang2fhir/create/multi",
            method="POST",
            json={
                "text": text,
                "version": version,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateMultiResponse,
                    parse_obj_as(
                        type_=CreateMultiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search(
        self, *, text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SearchResponse]:
        """
        Converts natural language text into FHIR search parameters.
        Automatically identifies the appropriate FHIR resource type and generates valid search query parameters.

        Supported resource types include: AllergyIntolerance, Appointment, CarePlan, CareTeam, Condition,
        Coverage, Device, DiagnosticReport, DocumentReference, Encounter, Goal, Immunization, Location,
        Medication, MedicationRequest, Observation, Organization, Patient, PlanDefinition, Practitioner,
        PractitionerRole, Procedure, Provenance, Questionnaire, QuestionnaireResponse, RelatedPerson,
        Schedule, ServiceRequest, Slot, and Specimen.

        Parameters
        ----------
        text : str
            Natural language text to convert into FHIR search parameters.
            The system will automatically identify the appropriate resource type and generate valid search parameters.

            Examples:
            - "Appointments between March 2-9, 2025" → Appointment search with date range
            - "Patients with diabetes" → Condition search with code parameter
            - "Active medication requests for metformin" → MedicationRequest search
            - "Lab results for creatinine" → DiagnosticReport search
            - "Dr. Smith's schedule" → Practitioner or Schedule search

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchResponse]
            Successfully generated FHIR search parameters
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lang2fhir/search",
            method="POST",
            json={
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchResponse,
                    parse_obj_as(
                        type_=SearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upload_profile(
        self, *, version: str, resource: str, profile: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Lang2FhirUploadProfileResponse]:
        """
        Upload a custom FHIR StructureDefinition profile for use with the lang2fhir service

        Parameters
        ----------
        version : str
            FHIR version that this profile implements

        resource : str
            Name for the custom resource profile (will be converted to lowercase)

        profile : str
            Base64 encoded JSON string of the FHIR StructureDefinition profile

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Lang2FhirUploadProfileResponse]
            Profile successfully uploaded
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lang2fhir/profile/upload",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "profile": profile,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirUploadProfileResponse,
                    parse_obj_as(
                        type_=Lang2FhirUploadProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def document(
        self,
        *,
        version: str,
        resource: DocumentRequestResource,
        content: str,
        file_type: DocumentRequestFileType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FhirResource]:
        """
        Extracts text from a document (PDF or image) and converts it into a structured FHIR resource

        Parameters
        ----------
        version : str
            FHIR version to use

        resource : DocumentRequestResource
            Type of FHIR resource to create (questionnaire and US Core questionnaireresponse profiles currently supported)

        content : str
            Base64 encoded file content

        file_type : DocumentRequestFileType
            MIME type of the file

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirResource]
            Successfully created FHIR resource from document
        """
        _response = await self._client_wrapper.httpx_client.request(
            "lang2fhir/document",
            method="POST",
            json={
                "version": version,
                "resource": resource,
                "content": content,
                "fileType": file_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirResource,
                    parse_obj_as(
                        type_=FhirResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CreateSummaryRequestFhirResources,
    CreateSummaryRequestMode,
    CreateSummaryResponse,
    CreateSummaryTemplateResponse,
    ErrorResponse,
    FhirBundle,
    FhirBundleEntryItem,
    FhirResource,
    SummaryDeleteTemplateResponse,
    SummaryGetTemplateResponse,
    SummaryListTemplatesResponse,
    SummaryTemplate,
    SummaryUpdateTemplateResponse,
)
from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError

__all__ = [
    "BadRequestError",
    "CreateSummaryRequestFhirResources",
    "CreateSummaryRequestMode",
    "CreateSummaryResponse",
    "CreateSummaryTemplateResponse",
    "ErrorResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirResource",
    "ForbiddenError",
    "InternalServerError",
    "NotFoundError",
    "SummaryDeleteTemplateResponse",
    "SummaryGetTemplateResponse",
    "SummaryListTemplatesResponse",
    "SummaryTemplate",
    "SummaryUpdateTemplateResponse",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawFhirProviderClient, RawFhirProviderClient
from .types.auth_method import AuthMethod
from .types.fhir_provider_delete_response import FhirProviderDeleteResponse
from .types.fhir_provider_list_response import FhirProviderListResponse
from .types.fhir_provider_remove_auth_config_response import FhirProviderRemoveAuthConfigResponse
from .types.fhir_provider_response import FhirProviderResponse
from .types.provider import Provider
from .types.role import Role
from .types.service_account_key import ServiceAccountKey

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FhirProviderClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFhirProviderClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFhirProviderClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFhirProviderClient
        """
        return self._raw_client

    def create(
        self,
        *,
        name: str,
        provider: Provider,
        auth_method: AuthMethod,
        base_url: str,
        description: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirProviderResponse:
        """
        Creates a new FHIR provider configuration with authentication credentials.

        Note: The "sandbox" provider type cannot be created via this API - it is managed internally.

        Parameters
        ----------
        name : str
            Display name for the FHIR provider

        provider : Provider

        auth_method : AuthMethod

        base_url : str
            Base URL of the FHIR server

        description : typing.Optional[str]
            Optional description of the FHIR provider

        client_id : typing.Optional[str]
            OAuth client ID (required for most auth methods)

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            FHIR provider created successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.create(
            name="Epic Sandbox",
            provider="athenahealth",
            auth_method="client_secret",
            base_url="https://fhir.epic.com/interconnect-fhir-oauth/api/FHIR/R4",
        )
        """
        _response = self._raw_client.create(
            name=name,
            provider=provider,
            auth_method=auth_method,
            base_url=base_url,
            description=description,
            client_id=client_id,
            client_secret=client_secret,
            service_account_key=service_account_key,
            role=role,
            scopes=scopes,
            request_options=request_options,
        )
        return _response.data

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> FhirProviderListResponse:
        """
        Retrieves a list of all active FHIR providers for the authenticated user.

        On shared instances, only sandbox providers are returned.
        Sandbox providers return FhirProviderSandboxInfo.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderListResponse
            FHIR providers retrieved successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def get(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderResponse:
        """
        Retrieves a specific FHIR provider configuration by its ID.

        Sandbox providers return FhirProviderSandboxInfo.
        On shared instances, only sandbox providers can be accessed.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            FHIR provider retrieved successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.get(
            fhir_provider_id="fhir_provider_id",
        )
        """
        _response = self._raw_client.get(fhir_provider_id, request_options=request_options)
        return _response.data

    def delete(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderDeleteResponse:
        """
        Soft deletes a FHIR provider by setting is_active to false.

        Note: Sandbox providers cannot be deleted.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderDeleteResponse
            FHIR provider deleted successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.delete(
            fhir_provider_id="fhir_provider_id",
        )
        """
        _response = self._raw_client.delete(fhir_provider_id, request_options=request_options)
        return _response.data

    def add_auth_config(
        self,
        fhir_provider_id: str,
        *,
        auth_method: AuthMethod,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        credential_expiry: typing.Optional[dt.datetime] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirProviderResponse:
        """
        Adds a new authentication configuration to an existing FHIR provider.
        This enables key rotation and multiple auth configurations per provider.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to add auth config to

        auth_method : AuthMethod

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        credential_expiry : typing.Optional[dt.datetime]
            Expiry time for JWT credentials (only applicable for JWT auth method)

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            Auth configuration added successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.add_auth_config(
            fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
            auth_method="client_secret",
        )
        """
        _response = self._raw_client.add_auth_config(
            fhir_provider_id,
            auth_method=auth_method,
            client_secret=client_secret,
            service_account_key=service_account_key,
            credential_expiry=credential_expiry,
            role=role,
            scopes=scopes,
            request_options=request_options,
        )
        return _response.data

    def set_active_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderResponse:
        """
        Sets which authentication configuration should be active for a FHIR provider.
        Only one auth config can be active at a time.

        If the specified auth config is already active, the request succeeds without
        making any changes and returns a message indicating the config is already active.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to set as active

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            Active auth configuration set successfully, or the config was already active.
            Check the message field to determine which case occurred.

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.set_active_auth_config(
            fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
            auth_config_id="auth-config-123",
        )
        """
        _response = self._raw_client.set_active_auth_config(
            fhir_provider_id, auth_config_id=auth_config_id, request_options=request_options
        )
        return _response.data

    def remove_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderRemoveAuthConfigResponse:
        """
        Removes an authentication configuration from a FHIR provider.
        Cannot remove the currently active auth configuration.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to remove

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderRemoveAuthConfigResponse
            Auth configuration removed successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir_provider.remove_auth_config(
            fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
            auth_config_id="auth-config-123",
        )
        """
        _response = self._raw_client.remove_auth_config(
            fhir_provider_id, auth_config_id=auth_config_id, request_options=request_options
        )
        return _response.data


class AsyncFhirProviderClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFhirProviderClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFhirProviderClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFhirProviderClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        name: str,
        provider: Provider,
        auth_method: AuthMethod,
        base_url: str,
        description: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirProviderResponse:
        """
        Creates a new FHIR provider configuration with authentication credentials.

        Note: The "sandbox" provider type cannot be created via this API - it is managed internally.

        Parameters
        ----------
        name : str
            Display name for the FHIR provider

        provider : Provider

        auth_method : AuthMethod

        base_url : str
            Base URL of the FHIR server

        description : typing.Optional[str]
            Optional description of the FHIR provider

        client_id : typing.Optional[str]
            OAuth client ID (required for most auth methods)

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            FHIR provider created successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.create(
                name="Epic Sandbox",
                provider="athenahealth",
                auth_method="client_secret",
                base_url="https://fhir.epic.com/interconnect-fhir-oauth/api/FHIR/R4",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            provider=provider,
            auth_method=auth_method,
            base_url=base_url,
            description=description,
            client_id=client_id,
            client_secret=client_secret,
            service_account_key=service_account_key,
            role=role,
            scopes=scopes,
            request_options=request_options,
        )
        return _response.data

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> FhirProviderListResponse:
        """
        Retrieves a list of all active FHIR providers for the authenticated user.

        On shared instances, only sandbox providers are returned.
        Sandbox providers return FhirProviderSandboxInfo.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderListResponse
            FHIR providers retrieved successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def get(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderResponse:
        """
        Retrieves a specific FHIR provider configuration by its ID.

        Sandbox providers return FhirProviderSandboxInfo.
        On shared instances, only sandbox providers can be accessed.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            FHIR provider retrieved successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.get(
                fhir_provider_id="fhir_provider_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(fhir_provider_id, request_options=request_options)
        return _response.data

    async def delete(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderDeleteResponse:
        """
        Soft deletes a FHIR provider by setting is_active to false.

        Note: Sandbox providers cannot be deleted.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderDeleteResponse
            FHIR provider deleted successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.delete(
                fhir_provider_id="fhir_provider_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(fhir_provider_id, request_options=request_options)
        return _response.data

    async def add_auth_config(
        self,
        fhir_provider_id: str,
        *,
        auth_method: AuthMethod,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        credential_expiry: typing.Optional[dt.datetime] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirProviderResponse:
        """
        Adds a new authentication configuration to an existing FHIR provider.
        This enables key rotation and multiple auth configurations per provider.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to add auth config to

        auth_method : AuthMethod

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        credential_expiry : typing.Optional[dt.datetime]
            Expiry time for JWT credentials (only applicable for JWT auth method)

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            Auth configuration added successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.add_auth_config(
                fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
                auth_method="client_secret",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_auth_config(
            fhir_provider_id,
            auth_method=auth_method,
            client_secret=client_secret,
            service_account_key=service_account_key,
            credential_expiry=credential_expiry,
            role=role,
            scopes=scopes,
            request_options=request_options,
        )
        return _response.data

    async def set_active_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderResponse:
        """
        Sets which authentication configuration should be active for a FHIR provider.
        Only one auth config can be active at a time.

        If the specified auth config is already active, the request succeeds without
        making any changes and returns a message indicating the config is already active.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to set as active

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderResponse
            Active auth configuration set successfully, or the config was already active.
            Check the message field to determine which case occurred.

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.set_active_auth_config(
                fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
                auth_config_id="auth-config-123",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set_active_auth_config(
            fhir_provider_id, auth_config_id=auth_config_id, request_options=request_options
        )
        return _response.data

    async def remove_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FhirProviderRemoveAuthConfigResponse:
        """
        Removes an authentication configuration from a FHIR provider.
        Cannot remove the currently active auth configuration.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to remove

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirProviderRemoveAuthConfigResponse
            Auth configuration removed successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir_provider.remove_auth_config(
                fhir_provider_id="1716d214-de93-43a4-aa6b-a878d864e2ad",
                auth_config_id="auth-config-123",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_auth_config(
            fhir_provider_id, auth_config_id=auth_config_id, request_options=request_options
        )
        return _response.data
